﻿using Exam.Infrastructure.Data.Entity;
using XF.Domain.Entities;
using XF.Domain.Repository;

namespace Hotline.Repository.SqlSugar.Extensions
{
    public static class ApiServiceExtension
    {
        public static void ToInsert<T,TActionRequest>(this T entity, TActionRequest actionRequest) where T : class, IEntity<string>, new()
        where TActionRequest : ActionRequest
        {
            entity.Id = Guid.NewGuid().ToString();

            if (entity is CreationEntity)
            {
                var creationEntity = entity as CreationEntity;
                if (creationEntity != null)
                {
                    creationEntity.CreatorId = actionRequest.UserId;
                    creationEntity.CreatorOrgId = actionRequest.OrgId;
                    creationEntity.CreatorOrgLevel = actionRequest.OrgLevel;
                    creationEntity.CreatorOrgName = actionRequest.OrgName;
                    creationEntity.CreatorName = actionRequest.UserName;
                    creationEntity.CreationTime = DateTime.Now;
                    creationEntity.AreaId = actionRequest.AreaId;

                    entity = creationEntity as T ?? entity;
                }

            }
        }

        public static void ToInsert<T, TActionRequest>(this List<T> entities,List<TActionRequest> actionRequests) where T : class, IEntity<string>, new()
        where TActionRequest : ActionRequest
        {
            foreach(var entity in entities)
            {
                var actionRequest = actionRequests.FirstOrDefault(x => x.Id == entity.Id);
                if (actionRequest != null)
                {
                    entity.ToInsert(actionRequest);
                }
            }
        }

        public static void ToUpdate<T, TActionRequest>(this T entity, TActionRequest actionRequest) where T : class, IEntity<string>, new()
        where TActionRequest : ActionRequest
        {
            if(entity is FullStateEntity)
            {
                var fullStateEntity = entity as FullStateEntity;
                if (fullStateEntity != null)
                {
                    fullStateEntity.CreatorId = actionRequest.UserId;
                    fullStateEntity.CreatorOrgId = actionRequest.OrgId;
                    fullStateEntity.CreatorOrgLevel = actionRequest.OrgLevel;
                    fullStateEntity.CreatorOrgName = actionRequest.OrgName;
                    fullStateEntity.CreatorName = actionRequest.UserName;
                    fullStateEntity.LastModificationTime = DateTime.Now;
                    fullStateEntity.AreaId = actionRequest.AreaId;

                    entity = fullStateEntity as T ?? entity;
                }
                
            }          
        }

        public static void ToUpdate<T, TActionRequest>(this List<T> entities, List<TActionRequest> actionRequests) where T : class, IEntity<string>, new()
       where TActionRequest : ActionRequest
        {
            foreach (var entity in entities)
            {
                var actionRequest = actionRequests.FirstOrDefault(x => x.Id == entity.Id);
                if (actionRequest != null)
                {
                    entity.ToUpdate(actionRequest);
                }
            }
        }
    }
}
